<?php
namespace App\Repositories;
use App\Models\Post;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Support\Facades\Redis;
class BeanRepository
{
    public function beanDetail($request)
    {
        try {
            $token = JWTAuth::decode(JWTAuth::getToken());
            $uid = $token['user']->uid;
            $sign = generateSign(['type' => $request['type'], 'uid' => $uid], config('customer.app_secret'));
            $url = config("customer.app_service_url") . '/user/v2/beanDetail';
            $array = [
                'json' => ['sign' => $sign, 'type' => $request['type'], 'uid' => $uid], 'query' => [], 'http_errors' => false, 'headers' => ['Authorization' => "Bearer " . JWTAuth::getToken()]
            ];
            return http($url, $array, 'get');
        } catch (\Exception $e) {
            Log::debug("beanDetail:" . $e->getMessage());
            return [];
        }
    }

    //获取优秀居民信息
    function excellentResidents($request)
    {
        $get_excellent = Redis::get('yesterday_excellent_residents');
        $excellent_residents = json_decode($get_excellent,true);

        if ($excellent_residents){
            //获取评论内容
            $post_ids = array_column($excellent_residents,'related_content_id');
            $comment_content = Post::select('id', 'title', 'content')->whereIn('id', $post_ids)->get();

            foreach ($comment_content->toArray() as $value) {
                foreach ($excellent_residents as $k=>$v) {
                    if (isset($v['related_content_id']) && $v['related_content_id'] == $value['id']) {
                        if (!empty($value['title'])) {
                            $excellent_residents[$k]['post_title'] = $value['title'];
                        }
                        if (empty($value['title']) && !empty($value['content'])){
                            $content = strip_tags($value['content']);
                            $excellent_residents[$k]['post_title'] = mb_substr($content, 0, 20);
                        }
                    }
                }
            }
            $content_author_id = array_column($excellent_residents,'content_author_id');
            $uids = implode(',', array_unique($content_author_id));
            $author_data = $this->getFollowMembersStatus($uids);
            if ($author_data){
                foreach ($excellent_residents as $k=>$v){
                    if(!isset($author_data[$v['content_author_id']])) continue;
                    $excellent_residents[$k]['follow_status'] = $author_data[$v['content_author_id']]['follow_status'];
                    $excellent_residents[$k]['username'] = $author_data[$v['content_author_id']]['username'];
                    $excellent_residents[$k]['avatar'] = $author_data[$v['content_author_id']]['avatar'];
                }
            }
        }

        return $excellent_residents;
    }

    public function rankingList($request)
    {
        if ($request['type'] == 0){//排行榜赚豆达人
            $yesterday = Carbon::yesterday()->toDateString();
            $all_beans = Redis::ZREVRANGEBYSCORE('user_rainbow_bean'.$yesterday, 100000000, 0, array('WITHSCORES'=>true,'limit' => array(0, 10)));
                $new_arr = [];
            if ($all_beans) {
                $i = 0;
                foreach ($all_beans as $key => $val) {
                    $new_arr[$i]['uid'] = $key;
                    $new_arr[$i]['count'] = intval($val);
                    $i++;
                }
                $uids = implode(',', array_column($new_arr, 'uid'));
                $user_data = $this->getFollowMembersStatus($uids);
                if ($user_data) {
                    foreach ($new_arr as $k => $v) {
                        if (!isset($user_data[$v['uid']])) continue;
                        $new_arr[$k]['follow_status'] = $user_data[$v['uid']]['follow_status'];
                        $new_arr[$k]['username'] = $user_data[$v['uid']]['username'];
                        $new_arr[$k]['avatar'] = $user_data[$v['uid']]['avatar'];
                    }
                }
            }
            return $new_arr;

        }elseif ($request['type'] == 1){//排行榜人脉达人
            $registered_mosts = Redis::get('yesterday_registered_most');
            if ($registered_mosts){
                $registered_most = json_decode($registered_mosts,true);
            }else{
                $registered_most = [];
            }
            if (count($registered_most) > 0) {
                $superior_uid = array_column($registered_most,'superior_uid');
                $uids = implode(',', array_unique($superior_uid));
                $user_data = $this->getFollowMembersStatus($uids);
                if ($user_data){
                    foreach ($registered_most as $k=>$v){
                        if(!isset($user_data[$v['superior_uid']])) continue;
                        $registered_most[$k]['count'] = intval($v['count']);
                        $registered_most[$k]['follow_status'] = $user_data[$v['superior_uid']]['follow_status'];
                        $registered_most[$k]['username'] = $user_data[$v['superior_uid']]['username'];
                        $registered_most[$k]['avatar'] = $user_data[$v['superior_uid']]['avatar'];
                    }
                }
            }
            return $registered_most;

        }else{//排行榜最佳作者
            $all_best_authors = Redis::get('yesterday_best_author');
            if ($all_best_authors){
                $all_best_author = json_decode($all_best_authors,true);
            }else{
                $all_best_author = [];
            }
            if (count($all_best_author) > 0) {
                $content_author_id = array_column($all_best_author,'content_author_id');
                $uids = implode(',', array_unique($content_author_id));
                $user_data = $this->getFollowMembersStatus($uids);
                if ($user_data){
                    foreach ($all_best_author as $k=>$v){
                        if(!isset($user_data[$v['content_author_id']])) continue;
                        $all_best_author[$k]['count'] = intval($v['count']);
                        $all_best_author[$k]['follow_status'] = $user_data[$v['content_author_id']]['follow_status'];
                        $all_best_author[$k]['username'] = $user_data[$v['content_author_id']]['username'];
                        $all_best_author[$k]['avatar'] = $user_data[$v['content_author_id']]['avatar'];
                    }
                }
            }
            return $all_best_author;
        }
    }


    function getFollowMembersStatus($uids) {
        try {
            $sign = generateSign(['uids' => $uids], config('customer.app_secret'));
            $url = config("customer.app_service_url").'/user/v2/member/getMemberIds';
            $array = [
                'json' => ['sign' => $sign,'uids' => $uids], 'query' => [], 'http_errors' => false,'headers'=>['Authorization'=>"Bearer ".JWTAuth::getToken()]
            ];
            return http($url,$array,'get');
        } catch (\Exception $e) {
            Log::debug($e->getMessage());
            return [];
        }

    }

    //星球首页
    public function starHome($request)
    {
        $today = Carbon::now()->toDateString();
        $token =  JWTAuth::decode(JWTAuth::getToken());
        $uid = $token['user']->uid;
        $bean = Redis::ZSCORE('user_rainbow_bean'.$today,$uid);

        $star_home = [];
        $user_bean = Redis::get('my_bean');
        Log::debug($user_bean);
        $user_bean = json_decode($user_bean,true);
        $yesterday_quantity_issued = Redis::get('yesterday_quantity_issued');
        Log::debug($user_bean);

        $my_bean = $this->get_user_bean($uid);
        $star_home['mybean']['user_all_bean'] = intval($my_bean['user_all_bean']) ?? 0;//用户总彩虹豆
        $star_home['mybean']['today_add_bean'] = intval($bean) ?? 0;//今日发放彩虹豆
        $star_home['mybean']['user_count'] = intval($user_bean['user_count']) ?? 0;//已入驻居民
        $star_home['mybean']['yesterday_add_user'] = intval($user_bean['yesterday_add_user']) ?? 0;//昨日新增居民
        $star_home['mybean']['yesterday_quantity_issued'] = intval($yesterday_quantity_issued) ?? 0;//昨日发放总彩虹豆

        $star_home['excellent_residents'] = $this->excellentResidents($request) ?? [];
        $star_home['daily_news'] = $this->getNews($request);
        $star_home['tips'] = $this->getPlatformContent($id = 1);

        return $star_home;

    }

    //获取某用户豆数
    function get_user_bean($uid)
    {
        try {
            $url = config("customer.app_service_url").'/user/v2/user_bean';
            $array = [
                'json' => ['uid' => $uid], 'query' => [], 'http_errors' => false,'headers'=>['Authorization'=>"Bearer ".JWTAuth::getToken()]
            ];
            return http($url,$array,'get');
        } catch (\Exception $e) {
            return [];
        }

    }

    //获取每日新闻
    function getNews($request) {
        try {
            $sign = generateSign([], config('customer.app_secret'));
            $url = config("customer.app_service_url").'/config/v2/starNews/lists';
            $array = [
                'json' => ['sign' => $sign], 'query' => [], 'http_errors' => false
            ];
            return http($url,$array,'get');
        } catch (\Exception $e) {
            Log::debug($e->getMessage());
            return [];
        }

    }
    //获取平台内容
    function getPlatformContent($id) {
        try {
            $url = config("customer.app_service_url").'/config/v2/platformContent/lists';
            $array = [
                'json' => ['id'=>$id], 'query' => [], 'http_errors' => false,'headers'=>['Authorization'=>"Bearer ".JWTAuth::getToken()]
            ];
            return http($url,$array,'get');
        } catch (\Exception $e) {
            Log::debug($e->getMessage());
            return [];
        }

    }

    public function starDetail($request)
    {
        $star_detail = [];
        $user_bean = Redis::get('my_bean');
        $user_bean = json_decode($user_bean,true);
        $yesterday_quantity_issued = Redis::get('yesterday_quantity_issued');
        $exchange = Redis::ZREVRANGEBYSCORE('post_purchase_message','+inf','-inf');
        foreach ($exchange as $k=>$v){
            $exchange[$k] = json_decode($v,true);
        }
        Log::debug($user_bean);
        Log::debug($yesterday_quantity_issued);
        Log::debug($exchange);

        $post_detail = new Post();
        if (isset($request['post_id']) && $request['post_id']){
            $post = $post_detail->
                join('post_data', 'post_data.post_id', '=', 'post.id')
                ->select('post.*','post_data.pv')
                ->find($request['post_id']);
            if ($post){
                $post->created_time = Carbon::parse($post->created_at)->diffForHumans();
                $imgs = [];
                foreach($post->imgs as $img){
                    $imgs[] = $img['img'];
                }
                unset($post->imgs);
                $post->imgs = $imgs;
                $post->topic = $post->topic();
                $star_detail['post'] = $post;
            }else{
                Log::debug($request['post_id']);
                return jsonError('该帖子已被删除');
            }
        }
        $user_info = $this->getUserInfo($invite_code = $request['invite_code']);
        Log::debug($request['invite_code']);
        Log::debug($user_info);
        if ($user_info){
            $star_detail['invite']['username'] = $user_info['username'] ?? '';
            $star_detail['invite']['avatar'] = $user_info['avatar'] ?? '';
            $star_detail['invite']['bean'] = config("customer.share_post_bean") ?? 0;
        }else{
            Log::debug($request['invite_code']);
            return jsonError('邀请码有误');
        }

        $star_detail['mybean']['user_count'] = intval($user_bean['user_count']) ?? 0;//已入驻居民
        $star_detail['mybean']['yesterday_add_user'] = intval($user_bean['yesterday_add_user']) ?? 0;//昨日新增居民
        $star_detail['mybean']['yesterday_quantity_issued'] = intval($yesterday_quantity_issued) ?? 0;//昨日发放总彩虹豆

        $star_detail['daily_news'] = $this->getNews($request);
        $star_detail['exchange'] = $exchange;

        return $star_detail;

    }

    //根据邀请码获取用户信息
    function getUserInfo($invite_code) {
        try {
            $sign = generateSign(['invite_code' => $invite_code], config('customer.app_secret'));
            $url = config("customer.app_service_url").'/user/v2/member/getInviteCodeMember';
            $array = [
                'json' => ['sign' => $sign,'invite_code' => $invite_code], 'query' => [], 'http_errors' => false,'headers'=>['Authorization'=>"Bearer ".JWTAuth::getToken()]
            ];
            return http($url,$array,'get');
        } catch (\Exception $e) {
            Log::debug($e->getMessage());
            return [];
        }

    }

}